#pragma once

#include <memory>

#include <wrl.h>
#include <d3d12.h>
#include <directxmath.h>
#include <directx/d3dx12.h>

class Win32DxInterface;
namespace mt
{
    class Window;
}

struct ID3D12CommandQueue;
struct ID3D12CommandAllocator;
struct ID3D12GraphicsCommandList;
struct IDXGISwapChain4;
struct ID3D12DescriptorHeap;
struct ID3D12Resource;
struct ID3D12Fence;
struct ID3D12RootSignature;
struct ID3D12PipelineState;

struct ID3D11Resource;
struct ID2D1Bitmap1;
struct ID2D1SolidColorBrush;
struct IDWriteTextFormat;

class Win32DxWindowInterface
{
public:
    Win32DxWindowInterface(Win32DxInterface* dxInterface);

    ~Win32DxWindowInterface();

    bool initialize(mt::Window* window);

    void onRender();

    //void afterPaint();

    //void resizeViewPort(unsigned width, unsigned height);

private:
    void loadPipeline(mt::Window* window);
    void loadAssets();

    void populateCommandList();
    void waitForPreviousFrame();
    void destroyWindow();

    void renderD2d();

    static const UINT FrameCount = 2;
    Win32DxInterface* mDxInterface{ nullptr };
    bool mInitialized{ false };

    struct Vertex
    {
        DirectX::XMFLOAT3 position;
        DirectX::XMFLOAT4 color;
    };

    CD3DX12_VIEWPORT mViewport;
    CD3DX12_RECT mScissorRect;

    Microsoft::WRL::ComPtr<ID3D12CommandAllocator> mCommandAllocator;
    Microsoft::WRL::ComPtr<ID3D12GraphicsCommandList> mCommandList;

    Microsoft::WRL::ComPtr<IDXGISwapChain4> mSwapChain;

    Microsoft::WRL::ComPtr<ID3D12RootSignature> mRootSignature;
    Microsoft::WRL::ComPtr<ID3D12PipelineState> mPipelineState;

    Microsoft::WRL::ComPtr<ID3D12DescriptorHeap> mRtvHeap;
    UINT mRtvDescriptorSize{ 0 };

    Microsoft::WRL::ComPtr<ID3D12Resource> mRenderTargets[FrameCount];
    Microsoft::WRL::ComPtr<ID3D11Resource> mWrappedBackBuffers[FrameCount];
    Microsoft::WRL::ComPtr<ID2D1Bitmap1> mD2dRenderTargets[FrameCount];

    Microsoft::WRL::ComPtr<ID2D1SolidColorBrush> mTextBrush;
    Microsoft::WRL::ComPtr<IDWriteTextFormat> mTextFormat;

    Microsoft::WRL::ComPtr<ID3D12Resource> mVertexBuffer;
    D3D12_VERTEX_BUFFER_VIEW mVertexBufferView{};

    Microsoft::WRL::ComPtr<ID3D12Fence> mFence;
    uint64_t mFenceValue = 0;
    uint64_t mFrameFenceValues[FrameCount] = {};
    HANDLE mFenceEvent{};

    uint64_t mFrameIndex{ 0 };
};

using XcbGlWindowInterfacePtr = std::unique_ptr<Win32DxWindowInterface>;